@@ -90,6 +90,7 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
 
             use std::env;
             use tokio::task;
+            use tokio::time::timeout;
             use checkout_core::cloud_pubsub;
             use checkout_core::schemars::JsonSchema;
             use checkout_core::dropshot;
@@ -163,39 +164,78 @@ fn impl_checkout_server(ast: &DeriveInput) -> TokenStream {
                     while subscription.client().is_running() {
                         println!("pulling payment processing messages");
 
-                        match subscription.get_messages::<IdWrapper>().await {
-                            Ok(messages) => {
-                                for (result, ack_id) in messages {
-                                    match result {
-                                        Ok(message) => {
-                                            println!("Received: {:?}", message);
-                                            match context_provider.new_context().await {
-                                                Ok(mut ctx) => {
-                                                    if !ctx.start_transaction().await.is_ok() {
-                                                        continue;
-                                                    }
-                                                    let processing_result = process_handler(&internal_ctx, &mut ctx, &message.0).await;
-                                                    if !processing_result.is_ok() {
-                                                        if !ctx.abort_transaction().await.is_ok() {
-                                                            // just log the failure to abort tx
-                                                            eprintln!("failed to commit tx");
+                        let pull = subscription.get_messages::<IdWrapper>();
+
+                        match timeout(Duration::from_secs(30), pull).await {
+                            Ok(result) => match result {
+                                Ok(messages) => {
+                                    for (result, ack_id) in messages {
+                                        match result {
+                                            Ok(message) => {
+                                                println!("Received: {:?}", message);
+                                                match context_provider.new_context().await {
+                                                    Ok(mut ctx) => {
+                                                        if !ctx.start_transaction().await.is_ok() {
+                                                            continue;
                                                         }
-                                                        continue;
+                                                        let processing_result = process_handler(&internal_ctx, &mut ctx, &message.0).await;
+                                                        if !processing_result.is_ok() {
+                                                            if !ctx.abort_transaction().await.is_ok() {
+                                                                // just log the failure to abort tx
+                                                                eprintln!("failed to commit tx");
+                                                            }
+                                                            continue;
+                                                        }
+                                                        if !ctx.commit_transaction().await.is_ok() {
+                                                            continue;
+                                                        }
+                                                        subscription.acknowledge_messages(vec![ack_id]).await;
                                                     }
-                                                    if !ctx.commit_transaction().await.is_ok() {
-                                                        continue;
-                                                    }
-                                                    subscription.acknowledge_messages(vec![ack_id]).await;
+                                                    Err(_) => {}
                                                 }
-                                                Err(_) => {}
                                             }
+                                            Err(e) => eprintln!("Failed to parse message: {}", e),
                                         }
-                                        Err(e) => eprintln!("Failed to parse message: {}", e),
                                     }
                                 }
-                            }
-                            Err(e) => eprintln!("Failed to pull PubSub messages: {}", e)
+                                Err(e) => eprintln!("Failed to pull PubSub messages: {}", e)
+                            },
+                            Err(_) => eprintln!("no response in 30 seconds")
                         }
+
+                        // match subscription.get_messages::<IdWrapper>().await {
+                        //     Ok(messages) => {
+                        //         for (result, ack_id) in messages {
+                        //             match result {
+                        //                 Ok(message) => {
+                        //                     println!("Received: {:?}", message);
+                        //                     match context_provider.new_context().await {
+                        //                         Ok(mut ctx) => {
+                        //                             if !ctx.start_transaction().await.is_ok() {
+                        //                                 continue;
+                        //                             }
+                        //                             let processing_result = process_handler(&internal_ctx, &mut ctx, &message.0).await;
+                        //                             if !processing_result.is_ok() {
+                        //                                 if !ctx.abort_transaction().await.is_ok() {
+                        //                                     // just log the failure to abort tx
+                        //                                     eprintln!("failed to commit tx");
+                        //                                 }
+                        //                                 continue;
+                        //                             }
+                        //                             if !ctx.commit_transaction().await.is_ok() {
+                        //                                 continue;
+                        //                             }
+                        //                             subscription.acknowledge_messages(vec![ack_id]).await;
+                        //                         }
+                        //                         Err(_) => {}
+                        //                     }
+                        //                 }
+                        //                 Err(e) => eprintln!("Failed to parse message: {}", e),
+                        //             }
+                        //         }
+                        //     }
+                        //     Err(e) => eprintln!("Failed to pull PubSub messages: {}", e)
+                        // }
                     }
                 });
             }
